"""
Abstract provider interface for any LLM backend.
"""

from __future__ import annotations

from abc import ABC, abstractmethod
from typing import Any, AsyncIterator, Dict, List, Literal, Optional, TypedDict

MessageRole = Literal["system", "user", "assistant", "tool"]


class Message(TypedDict):
    role: MessageRole
    content: str


class ToolCall(TypedDict):
    id: str
    type: Literal["function"]
    function: Dict[str, Any]  # {"name": str, "arguments": str}


class Provider(ABC):
    """Protocol every LLM backend must satisfy."""

    @abstractmethod
    async def chat(
        self,
        messages: List[Message],
        tools: Optional[List[Dict[str, Any]]] = None,
    ) -> AsyncIterator[str]:
        """
        Stream the assistant response (delta text or final text).
        If tools are provided, the provider may return a tool_calls
        block instead of plain text.
        """
        ...